// 
// Copyright (C) 2006, NinjaTrader LLC <www.ninjatrader.com>.
// NinjaTrader reserves the right to modify or overwrite this NinjaScript component with each release.
//

#region Using declarations
using System;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.ComponentModel;
using System.Xml.Serialization;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
	/// <summary>
	/// The STOchRSIMod is an oscillator similar in computation to the stochastic measure, except instead of price values as input, the STORsi uses RSI values. The STORsi computes the current position of the RSI relative to the high and low RSI values over a specified number of days. The intent of this measure, designed by Tushard Chande and Stanley Kroll, is to provide further information about the overbought/oversold nature of the RSI. The STORsi ranges between 0 and 100. Values above 80 are generally seen to identify overbought levels and values below 20 are considered to indicate oversold conditions.
	/// </summary>
	[Description("The STOchRSIMod is an oscillator similar in computation to the stochastic measure, except instead of price values as input, the STORsi uses RSI values. The STORsi computes the current position of the RSI relative to the high and low RSI values over a specified number of days. The intent of this measure, designed by Tushard Chande and Stanley Kroll, is to provide further information about the overbought/oversold nature of the RSI. The STORsi ranges between 0.0 and 1.0. Values above 0.8 are generally seen to identify overbought levels and values below 0.2 are considered to indicate oversold conditions.")]
	[Gui.Design.DisplayName("STOchRSIMod")]
	public class STOchRSIMod1 : Indicator
	{
		#region Variables
		private int			periodRsi		= 21;
		private int			fastMaPeriod	=5;
		private int			slowMaPeriod	=8;
		private int 		lookBack		=13;

		
		private DataSeries minimum;
		private DataSeries maximum;
		private DataSeries rsi;
		private DataSeries storsi;
		private DataSeries PivotLow;
		private DataSeries PivotLowBars;
		private DataSeries PivotHigh;
		private DataSeries PivotHighBars;
		
		#endregion

		/// <summary>
		/// This method is used to configure the indicator and is called once before any bar data is loaded.
		/// </summary>
		protected override void Initialize()
		{
			Add(new Plot(Color.Blue, "SK"));
			Add(new Plot(Color.Red,  "SD"));
			
			Add(new Line(Color.Gray,  80, "Overbought"));
			Add(new Line(Color.Gray,  20, "Oversold"));
			
			Overlay				= false;
			PriceTypeSupported	= false;
			DrawOnPricePanel 	= false;
			
			minimum = new DataSeries(this);
			maximum = new DataSeries(this);
			rsi = new DataSeries(this);
			storsi = new DataSeries(this);
			PivotLow = new DataSeries(this);
			PivotLowBars = new DataSeries(this);
			PivotHigh = new DataSeries(this);
			PivotHighBars = new DataSeries(this);			
		}

		/// <summary>
		/// Called on each bar update event (incoming tick)
		/// </summary>
		protected override void OnBarUpdate()
		{
			if ((CurrentBar < periodRsi) || (CurrentBar < SlowMaPeriod) || (CurrentBar < FastMaPeriod))
				return;

			minimum.Set( MIN(RSI(PeriodRsi,1), LookBack)[0]);
			maximum.Set( MAX(RSI(PeriodRsi,1), LookBack)[0]);
			rsi.Set( RSI( PeriodRsi,1)[0]);
			double denominator = (maximum[0]-minimum[0]);
			if (denominator == 0) denominator = 1;							// Prevent divide by zero error.
			storsi.Set( (rsi[0]-minimum[0]) / denominator * 100);
//Print(String.Format("5: Bar={0} minimum={1} maximum={2} rsi={3} storsi={4}", CurrentBar, minimum[0], maximum[0], rsi[0], storsi[0] ));

			SK.Set(SMA(storsi, SlowMaPeriod)[0]);
			SD.Set(SMA(SK,FastMaPeriod)[0]);
//Print(String.Format("7: Bar={0} SK={1} SD={2}", CurrentBar, SK[0], SD[0] ));
		
		}

		#region Properties
		/// <summary>
		/// </summary>
		[Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries SK
        {
            get { return Values[0]; }
        }
		
		[Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries SD
        {
            get { return Values[1]; }
        }
		
		[Description("Numbers of bars used for calculations")]
		[Category("Parameters")]
		public int PeriodRsi
		{
			get { return periodRsi; }
			set { periodRsi = Math.Max(1, value); }
		}
		[Description("FAst Moving Average")]
		[Category("Parameters")]
		public int FastMaPeriod
		{
			get { return fastMaPeriod; }
			set { fastMaPeriod = Math.Max(1, value); }
		}
		[Description("Slow Moving Average")]
		[Category("Parameters")]
		public int SlowMaPeriod
		{
			get { return slowMaPeriod; }
			set { slowMaPeriod = Math.Max(1, value); }
		}
		[Description("Look back Period")]
		[Category("Parameters")]
		public int LookBack
		{
			get { return lookBack; }
			set { lookBack = Math.Max(1, value); }
		}
			
		#endregion
	}
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private STOchRSIMod1[] cacheSTOchRSIMod1 = null;

        private static STOchRSIMod1 checkSTOchRSIMod1 = new STOchRSIMod1();

        /// <summary>
        /// The STOchRSIMod is an oscillator similar in computation to the stochastic measure, except instead of price values as input, the STORsi uses RSI values. The STORsi computes the current position of the RSI relative to the high and low RSI values over a specified number of days. The intent of this measure, designed by Tushard Chande and Stanley Kroll, is to provide further information about the overbought/oversold nature of the RSI. The STORsi ranges between 0.0 and 1.0. Values above 0.8 are generally seen to identify overbought levels and values below 0.2 are considered to indicate oversold conditions.
        /// </summary>
        /// <returns></returns>
        public STOchRSIMod1 STOchRSIMod1(int fastMaPeriod, int lookBack, int periodRsi, int slowMaPeriod)
        {
            return STOchRSIMod1(Input, fastMaPeriod, lookBack, periodRsi, slowMaPeriod);
        }

        /// <summary>
        /// The STOchRSIMod is an oscillator similar in computation to the stochastic measure, except instead of price values as input, the STORsi uses RSI values. The STORsi computes the current position of the RSI relative to the high and low RSI values over a specified number of days. The intent of this measure, designed by Tushard Chande and Stanley Kroll, is to provide further information about the overbought/oversold nature of the RSI. The STORsi ranges between 0.0 and 1.0. Values above 0.8 are generally seen to identify overbought levels and values below 0.2 are considered to indicate oversold conditions.
        /// </summary>
        /// <returns></returns>
        public STOchRSIMod1 STOchRSIMod1(Data.IDataSeries input, int fastMaPeriod, int lookBack, int periodRsi, int slowMaPeriod)
        {
            checkSTOchRSIMod1.FastMaPeriod = fastMaPeriod;
            fastMaPeriod = checkSTOchRSIMod1.FastMaPeriod;
            checkSTOchRSIMod1.LookBack = lookBack;
            lookBack = checkSTOchRSIMod1.LookBack;
            checkSTOchRSIMod1.PeriodRsi = periodRsi;
            periodRsi = checkSTOchRSIMod1.PeriodRsi;
            checkSTOchRSIMod1.SlowMaPeriod = slowMaPeriod;
            slowMaPeriod = checkSTOchRSIMod1.SlowMaPeriod;

            if (cacheSTOchRSIMod1 != null)
                for (int idx = 0; idx < cacheSTOchRSIMod1.Length; idx++)
                    if (cacheSTOchRSIMod1[idx].FastMaPeriod == fastMaPeriod && cacheSTOchRSIMod1[idx].LookBack == lookBack && cacheSTOchRSIMod1[idx].PeriodRsi == periodRsi && cacheSTOchRSIMod1[idx].SlowMaPeriod == slowMaPeriod && cacheSTOchRSIMod1[idx].EqualsInput(input))
                        return cacheSTOchRSIMod1[idx];

            STOchRSIMod1 indicator = new STOchRSIMod1();
            indicator.SetUp();
            indicator.BarsRequired = BarsRequired;
            indicator.CalculateOnBarClose = CalculateOnBarClose;
            indicator.Input = input;
            indicator.FastMaPeriod = fastMaPeriod;
            indicator.LookBack = lookBack;
            indicator.PeriodRsi = periodRsi;
            indicator.SlowMaPeriod = slowMaPeriod;

            STOchRSIMod1[] tmp = new STOchRSIMod1[cacheSTOchRSIMod1 == null ? 1 : cacheSTOchRSIMod1.Length + 1];
            if (cacheSTOchRSIMod1 != null)
                cacheSTOchRSIMod1.CopyTo(tmp, 0);
            tmp[tmp.Length - 1] = indicator;
            cacheSTOchRSIMod1 = tmp;
            Indicators.Add(indicator);

            return indicator;
        }

    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// The STOchRSIMod is an oscillator similar in computation to the stochastic measure, except instead of price values as input, the STORsi uses RSI values. The STORsi computes the current position of the RSI relative to the high and low RSI values over a specified number of days. The intent of this measure, designed by Tushard Chande and Stanley Kroll, is to provide further information about the overbought/oversold nature of the RSI. The STORsi ranges between 0.0 and 1.0. Values above 0.8 are generally seen to identify overbought levels and values below 0.2 are considered to indicate oversold conditions.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.STOchRSIMod1 STOchRSIMod1(int fastMaPeriod, int lookBack, int periodRsi, int slowMaPeriod)
        {
            return _indicator.STOchRSIMod1(Input, fastMaPeriod, lookBack, periodRsi, slowMaPeriod);
        }

        /// <summary>
        /// The STOchRSIMod is an oscillator similar in computation to the stochastic measure, except instead of price values as input, the STORsi uses RSI values. The STORsi computes the current position of the RSI relative to the high and low RSI values over a specified number of days. The intent of this measure, designed by Tushard Chande and Stanley Kroll, is to provide further information about the overbought/oversold nature of the RSI. The STORsi ranges between 0.0 and 1.0. Values above 0.8 are generally seen to identify overbought levels and values below 0.2 are considered to indicate oversold conditions.
        /// </summary>
        /// <returns></returns>
        public Indicator.STOchRSIMod1 STOchRSIMod1(Data.IDataSeries input, int fastMaPeriod, int lookBack, int periodRsi, int slowMaPeriod)
        {
            return _indicator.STOchRSIMod1(input, fastMaPeriod, lookBack, periodRsi, slowMaPeriod);
        }

    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// The STOchRSIMod is an oscillator similar in computation to the stochastic measure, except instead of price values as input, the STORsi uses RSI values. The STORsi computes the current position of the RSI relative to the high and low RSI values over a specified number of days. The intent of this measure, designed by Tushard Chande and Stanley Kroll, is to provide further information about the overbought/oversold nature of the RSI. The STORsi ranges between 0.0 and 1.0. Values above 0.8 are generally seen to identify overbought levels and values below 0.2 are considered to indicate oversold conditions.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.STOchRSIMod1 STOchRSIMod1(int fastMaPeriod, int lookBack, int periodRsi, int slowMaPeriod)
        {
            return _indicator.STOchRSIMod1(Input, fastMaPeriod, lookBack, periodRsi, slowMaPeriod);
        }

        /// <summary>
        /// The STOchRSIMod is an oscillator similar in computation to the stochastic measure, except instead of price values as input, the STORsi uses RSI values. The STORsi computes the current position of the RSI relative to the high and low RSI values over a specified number of days. The intent of this measure, designed by Tushard Chande and Stanley Kroll, is to provide further information about the overbought/oversold nature of the RSI. The STORsi ranges between 0.0 and 1.0. Values above 0.8 are generally seen to identify overbought levels and values below 0.2 are considered to indicate oversold conditions.
        /// </summary>
        /// <returns></returns>
        public Indicator.STOchRSIMod1 STOchRSIMod1(Data.IDataSeries input, int fastMaPeriod, int lookBack, int periodRsi, int slowMaPeriod)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.STOchRSIMod1(input, fastMaPeriod, lookBack, periodRsi, slowMaPeriod);
        }

    }
}
#endregion
